"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppConfigService = void 0;
const effect_1 = require("effect");
const ConfigError_1 = require("effect/ConfigError");
const printer_1 = require("@effect/printer");
const Render_1 = require("@effect/printer/Render");
const types_1 = require("./types");
const mapBrandOrFail = (brand) => (config) => config.pipe(effect_1.Config.mapOrFail((v) => brand.either(v).pipe(effect_1.Either.mapLeft(brandErrorToConfigError))));
const brandErrorToConfigError = (e) => (0, ConfigError_1.InvalidData)([], e.map((_) => _.message).join(' '));
const jiraCloudAuthConfig = effect_1.Config.all({
    jiraApiToken: effect_1.Config.string('JIRA_API_TOKEN').pipe(mapBrandOrFail(types_1.JiraApiToken)),
    jiraUserEmail: effect_1.Config.string('JIRA_USER_EMAIL').pipe(mapBrandOrFail(types_1.JiraUserEmail)),
}).pipe(effect_1.Config.map(types_1.JiraCloudAuth));
const jiraDataCenterAuthConfig = effect_1.Config.all({
    jiraPat: effect_1.Config.string('JIRA_PAT').pipe(mapBrandOrFail(types_1.JiraPat)),
}).pipe(effect_1.Config.map(types_1.JiraDataCenterAuth));
const appConfig = effect_1.Config.all({
    jiraApiUrl: effect_1.Config.string('JIRA_API_URL').pipe(mapBrandOrFail(types_1.JiraApiUrl)),
    defaultJiraKeyPrefix: effect_1.Config.option(effect_1.Config.string('JIRA_KEY_PREFIX').pipe(mapBrandOrFail(types_1.JiraKeyPrefix))),
    jiraAuth: effect_1.Config.orElse(jiraCloudAuthConfig, () => jiraDataCenterAuthConfig),
});
const collectConfigErrorMessagesReducer = (() => {
    const collectMessage = (_, _path, message) => printer_1.Doc.text(message);
    return {
        andCase: (_, left, right) => printer_1.Doc.vcat([left, right]),
        orCase: (_, left, right) => printer_1.Doc.vcat([printer_1.Doc.indent(left, 2), printer_1.Doc.text('or'), printer_1.Doc.indent(right, 2)]),
        invalidDataCase: (_, path, message) => printer_1.Doc.text(`Expected valid input for ${path.join('.')} (${message})`),
        missingDataCase: collectMessage,
        sourceUnavailableCase: collectMessage,
        unsupportedCase: collectMessage,
    };
})();
class AppConfigService extends effect_1.Context.Tag('AppConfigService')() {
}
exports.AppConfigService = AppConfigService;
AppConfigService.Live = effect_1.Layer.succeed(AppConfigService, AppConfigService.of({
    getAppConfig: appConfig.pipe(effect_1.Effect.mapError((0, effect_1.flow)(effect_1.ConfigError.reduceWithContext(void 0, collectConfigErrorMessagesReducer), Render_1.prettyDefault, (message) => (0, types_1.AppConfigError)({ message })))),
}));
