"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cliEffect = void 0;
const effect_1 = require("effect");
const cli_1 = require("@effect/cli");
const Function_1 = require("effect/Function");
const Args = __importStar(require("@effect/cli/Args"));
const Command = __importStar(require("@effect/cli/Command"));
const Options = __importStar(require("@effect/cli/Options"));
const Span = __importStar(require("@effect/cli/HelpDoc/Span"));
const HelpDoc = __importStar(require("@effect/cli/HelpDoc"));
const core_1 = require("./core");
const packageJson = __importStar(require("../package.json"));
const types_1 = require("./types");
const url_opener_1 = require("./url-opener");
// for version and help
const gitJiraBranch = (0, effect_1.pipe)(Command.make('git-create-jira-branch', {}));
const createCommand = (0, effect_1.pipe)(Command.make('create', {
    options: Options.all({
        baseBranch: Options.withDescription(Options.optional(Options.withAlias(Options.text('base'), 'b')), 'Base revision to create the new branch from (a branch name, tag or commit SHA)'),
        reset: Options.withDescription(Options.withAlias(Options.boolean('reset'), 'r'), 'Reset the branch if it already exists'),
    }),
    jiraKey: Args.withDescription(Args.text({ name: 'jira-key' }), 'The Jira ticket key to create a branch for (e.g. FOOX-1234)'),
}, ({ options, jiraKey }) => {
    return effect_1.Effect.flatMap((0, core_1.gitCreateJiraBranch)(jiraKey, options.baseBranch, options.reset), (0, Function_1.compose)((0, types_1.matchGitCreateJiraBranchResult)({
        onCreatedBranch: (branch) => `Successfully created branch: '${branch}'`,
        onSwitchedBranch: (branch) => `Switched to already existing branch: '${branch}'`,
        onResetBranch: (branch) => `Reset branch: '${branch}'`,
    }), effect_1.Console.log));
}), Command.withDescription(`
Fetches the given Jira ticket and creates an aproriately named branch for it.
The branch type (bug or feat) is determined by the ticket type. The branch name
is based on the ticket summary.`));
const openCommand = (0, effect_1.pipe)(Command.make('open', {
    jiraKey: Args.withDescription(Args.optional(Args.text({ name: 'jira-key' })), 'The Jira ticket key to create a branch for (e.g. FOOX-1234)'),
}, ({ jiraKey }) => (0, effect_1.pipe)(jiraKey, effect_1.Option.match({
    onSome: (jiraKey) => (0, core_1.ticketUrl)(jiraKey),
    onNone: () => (0, core_1.ticketUrlForCurrentBranch)(),
}), effect_1.Effect.tap((url) => effect_1.Console.log(`Opening ticket url '${url}' in your default browser...`)), effect_1.Effect.flatMap(url_opener_1.openUrl))), Command.withDescription(`
Opens the given Jira ticket in your default browser. If no ticket is given the
jira ticket for the current branch is opened.`));
const mainCommand = gitJiraBranch.pipe(Command.withSubcommands([createCommand, openCommand]));
const cli = {
    name: packageJson.name,
    version: packageJson.version,
    summary: Span.text(packageJson.description),
};
const cliEffect = (args) => Command.run(mainCommand, cli)(args).pipe(effect_1.Effect.catchIf(cli_1.ValidationError.isValidationError, (e) => cli_1.ValidationError.isInvalidValue(e) // Not printed by the cli yet
    ? printDocs(HelpDoc.p(Span.error(HelpDoc.getSpan(e.error))))
    : effect_1.Effect.succeed(undefined)), effect_1.Effect.catchAll((e) => printDocs(HelpDoc.p(Span.error(e.message)))));
exports.cliEffect = cliEffect;
const printDocs = (doc) => effect_1.Console.log(HelpDoc.toAnsiText(doc));
